import io
import time
from typing import Self, Type

import pydantic
import telebot
from sqlalchemy.future import Engine

from .. import database, cryptography


class Account(pydantic.BaseModel):
    name: str
    login: str
    passwd: str

    @classmethod
    def from_tuple(cls: Type[Self], tuple_: tuple[str, str, str]) -> Self:
        return cls(name=tuple_[0], login=tuple_[1], passwd=tuple_[2])

    def as_tuple(self: Self) -> tuple[str, str, str]:
        return (self.name, self.login, self.passwd)


class _Accounts(pydantic.BaseModel):
    accounts: list[Account] = pydantic.Field(default_factory=list)


def _accounts_list_to_json(accounts: list[tuple[str, str, str]]) -> str:
    accounts = _Accounts(accounts=[Account.from_tuple(i) for i in accounts])
    return accounts.json()


def json_to_accounts(json_: str) -> list[tuple[str, str, str]]:
    accounts = _Accounts.parse_raw(json_)
    return [i.as_tuple() for i in accounts.accounts]


Message = telebot.types.Message


def send_tmp_message(
    bot: telebot.TeleBot, chat_id: telebot.types.Message, text: str, timeout: int = 5
) -> None:
    bot_mes = bot.send_message(chat_id, text, parse_mode="MarkdownV2")
    time.sleep(timeout)
    bot.delete_message(chat_id, bot_mes.id)


def base_handler(
    bot: telebot.TeleBot, mes: Message, prev_mes: Message | None = None
) -> None:
    bot.delete_message(mes.chat.id, mes.id)
    if prev_mes is not None:
        bot.delete_message(prev_mes.chat.id, prev_mes.id)


def get_all_accounts(
    engine: Engine, user_id: int, master_pass: str
) -> list[tuple[str, str, str]]:
    accounts: list[tuple[str, str, str]] = []
    master_pass = master_pass.encode("utf-8")
    for account_name in database.get.get_accounts(engine, user_id):
        salt, enc_login, enc_passwd = database.get.get_account_info(
            engine, user_id, account_name
        )
        login, passwd = cryptography.other_accounts.decrypt_account_info(
            enc_login, enc_passwd, master_pass, salt
        )
        accounts.append((account_name, login, passwd))
    return accounts


def accounts_to_json(accounts: list[tuple[str, str, str]]) -> io.StringIO:
    file = io.StringIO(_accounts_list_to_json(accounts))
    file.name = "passwords.json"
    return file
